; min.mdp - used as input into grompp to generate em.tpr
integrator	= steep		          ; Algorithm (steep = steepest descent minimization)
emtol		= 1000.0  	          ; Stop minimization when the maximum force < 1000.0 kJ/mol/nm
emstep      = 0.01                ; Energy step size
nsteps		= {{ nsteps_minim }}  ; Maximum number of (minimization) steps to perform

; Parameters describing how to find the neighbors of each atom and how to calculate the interactions
nstlist		    = 1		    ; Frequency to update the neighbor list and long range forces
cutoff-scheme   = Verlet    ; 
ns_type		    = grid		; Method to determine neighbor list (simple, grid)
coulombtype	    = PME		; Treatment of long range electrostatic interactions
rcoulomb	    = 1.0		; Short-range electrostatic cut-off
rvdw		    = 1.0		; Short-range Van der Waals cut-off
pbc		        = xyz 		; Periodic Boundary Conditions (yes/no)