[
    {"cmd": "gmx pdb2gmx -f protein.pdb -o protein.gro -water tip3p -ff amber99sb-ildn", "prompts":[]},
    {"cmd": "gmx editconf -f protein.gro -o protein_box.gro -c yes -d 1.0 -bt cubic", "prompts":[]},
    {"cmd": "gmx solvate -cp protein_box.gro -o protein_solv.gro -p topol.top", "prompts":[]},
    {"cmd": "gmx grompp -f ions.mdp -c protein_solv.gro -p topol.top -o ions.tpr {% if charged %} -maxwarn 1 {% endif %}", "prompts":[]},
    {"cmd": "gmx genion -s ions.tpr -o protein_solv_ions.gro -p topol.top -pname NA -nname CL -neutral yes", "prompts":[{% if charged %} "SOL" {% endif %}]},
    {"cmd": "gmx grompp -f energy_minimization.mdp -c protein_solv_ions.gro -p topol.top -o em.tpr", "prompts":[]},
    {"cmd": "gmx mdrun -deffnm em -v yes", "prompts":[]},
    {"cmd": "gmx grompp -f simulation.mdp -c em.gro -r em.gro -p topol.top -o solvated_protein.tpr", "prompts":[]},
    {"cmd": "gmx mdrun -deffnm solvated_protein -v yes", "prompts":[]},
    {"cmd": "gmx trjconv -f solvated_protein.trr -s solvated_protein.tpr -o trajectory.xtc", "prompts":[ "{{visualized_section}}"] }
]