[
    {"cmd": "surfaceFeatures", "prompts": []},
    {"cmd": "blockMesh", "prompts":[]},
    {% if n_cores > 1 %} {"cmd": "decomposePar -copyZero", "prompts":[]},
    {"cmd": "mpirun --allow-run-as-root -np {{ n_cores }} snappyHexMesh -overwrite -parallel", "prompts":[]},
    {"cmd": "reconstructParMesh -constant", "prompts":[]},
    {% else %} {"cmd": "snappyHexMesh -overwrite", "prompts":[]},
    {% endif %}
    {"cmd": "potentialFoam", "prompts":[]},
    {% if n_cores > 1 %} 
    {"cmd": "mpirun --allow-run-as-root -np {{ n_cores }} simpleFoam -parallel", "prompts":[]},
    {"cmd": "reconstructPar", "prompts":[]}
    {% else %} {"cmd": "simpleFoam", "prompts":[]}
    {% endif %}
]