; minim.mdp - used as input into grompp to generate em.tpr
integrator	    = steep		         ; Algorithm (steep = steepest descent minimization)
emtol		    = 1000  	         ; Stop minimization when the maximum force < 1000.0 kJ/mol/nm
emstep          = 0.01               ; Energy step size
nsteps		    = {{ nsteps_minim }} ; Maximum number of (minimization) steps to perform
cutoff-scheme   = Verlet             ; Cut-off scheme used for short-range interactions
ns_type         = grid               ; Method to determine neighbor list (simple, grid)
coulombtype	    = PME		         ; Treatment of long range electrostatic interactions
rcoulomb	    = 1.0		         ; Short-range electrostatic cut-off
pbc		        = xyz 		         ; Periodic Boundary Conditions (yes/no)
constraints     = h-bonds            ; apply constraints to hydrogen bonds
